#!/usr/bin/env python
"""
    gladex - Generate source code from Glade XML files.

    Copyright (C) 2007 See AUTHORS list for copyright holders

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import commands
import time
from threading import Timer
from time import sleep
import sys
import os
import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import inspect
import gladex_functions as gxf
import plugins

global_data = {}
global_data["main_window_maped"]=False

class fallbackplugin:
	def info(self):
		pass
	info.name="Fallback"
	info.version=1
	info.compatibility=0.4
	info.description="This is a fallback plugin"
	info.plugin_type="generator"


	def widget(self):
		global plugdir
		return gtk.Label("Error with plugin!")
	#

	def execute():
		return "Error with Plugin!"
	#


	def on_glade_file_change(widget):
		pass
	#

	def on_directory_change(widget):
		pass
#


def load_plugins():
	plugins_list = []
	plugs = dir(plugins)
	for i in range(len(plugs)):
		if plugs[i][0:2]=="__":continue
		try:
			plugins_list.append(__import__("plugins/"+inspect.getmodulename(plugs[i]+".py")))
			t=plugins_list[-1].widget
			t=plugins_list[-1].execute
			t=plugins_list[-1].info
			t=plugins_list[-1].on_directory_change
			t=plugins_list[-1].on_glade_file_change
		except:
			plugins_list.append(fallbackplugin())
	return plugins_list




def gtk_main_quit(widget, data, wtree):
	gtk.gdk.threads_leave()
	gtk.main_quit()
	pass


def on_MainWindow_map_event(widget, data, wtree):
	global global_data
	if global_data["main_window_maped"]==True:
		return
	global_data["main_window_maped"] = True
	global_data["plugins"] = []
	plugin_list = load_plugins()
	
	global_data["tree"] = wtree
	
	plugin_list_combo = wtree.get_widget("plugin_list")
	
	d = os.getcwd()
	global_data["threadrunning"] = False
	home = global_data["home"]=commands.getoutput("cd ~;pwd")
	wtree.get_widget("glade_file_chooser").set_current_folder_uri(home)
	wtree.get_widget("output_directory_chooser").set_current_folder_uri(home)
	commands.getoutput("cd "+d)
	
	listStore = gtk.ListStore(str)
	listStore.append(["Select Plugin ..."])
	for plug in plugin_list:
		global_data["plugins"].append(plug)
		#print plug.info.name
		plugin_list_combo.append_text(plug.info.name)
		listStore.append([plug.info.name])
		#plugin_list_combo.append_text("hello")
	
	
	#plugin_list_combo.set_model(listStore)
	print plugin_list_combo.get_model()
	
	plugin_list_combo.set_active(0)
	
	glade_file_filter = gtk.FileFilter()
	glade_file_filter.add_mime_type("glade")
	glade_file_filter.add_pattern("*.glade")
	wtree.get_widget("glade_file_chooser").set_filter(glade_file_filter)
	pass
#
def send_status_message(messages):
	bar = global_data["tree"].get_widget("statusbar")
	global_data["threadrunning"]=True
	for x in messages:
		id = bar.get_context_id(x)
		bar.push(id, x)
		sleep(2)
		bar.pop(id)
	global_data["threadrunning"]=False
def show_message(msg, status_message=False):
	global global_data
	if status_message:
		msg = msg.replace("\t","   ")
		if global_data["threadrunning"] == False:
			t = Timer(0, send_status_message, [msg.split("\n")])
			t.start()
		return
	message_window = global_data["tree"].get_widget("message_window")
	message_window.set_markup(msg)
	responce = message_window.run()
	if responce == gtk.RESPONSE_OK or responce==gtk.RESPONSE_DELETE_EVENT:
		message_window.hide()

def on_plugin_list_changed(widget, data, wtree):
	global global_data
	#print "plugin list changed"
	load_box = wtree.get_widget("load_box")
	for x in load_box.get_children():
		load_box.remove(x)
	activate_plugin_index = widget.get_active() -1 # minus 1 because 0 is 'select plugin ...' in the combo list
	#print activate_plugin_index
	if activate_plugin_index < 0: return
	newWidget = global_data["plugins"][activate_plugin_index].widget()
	newWidget.unparent()
	load_box.add(newWidget)
	newWidget.show_all()
	pass



def on_info_button_clicked(widget, data, wtree):
	plug = get_selected_plugin()
	if plug == None:
		message = "please select a plugin"
	else:
		message = plug.info.description
	show_message(message)


def get_selected_plugin():
	global global_data
	plugin_combo = global_data["tree"].get_widget("plugin_list")
	activate_plugin_index = plugin_combo.get_active() - 1
	if activate_plugin_index < 0:
		return None
	else:
		return global_data["plugins"][activate_plugin_index]
	
def on_execute_clicked(widget, data, wtree):
	global global_data
	## TODO:
	# check if a glade file and a plugin is chosen
	c1 = wtree.get_widget("output_directory_chooser").get_filename()
	c2 = wtree.get_widget("glade_file_chooser").get_filename()
	if None in [c1, c2]:
		show_message("please choose a glade file",True)
		return
	
	
	# calls execute function from plugin
	plug = get_selected_plugin()
	if plug == None:
		message = "please select a plugin"
	else:
		#execute plugin code
		message = plug.execute()
	# display message
	show_message(message, True)
	pass


def on_about_menu_activate(widget, data, wtree):
	about = wtree.get_widget("about_gladex")
	responce = about.run()
	if responce == gtk.RESPONSE_CANCEL or responce==gtk.RESPONSE_DELETE_EVENT:
		about.hide()
	pass
#

def on_help_menu_activate(widget, data, wtree):
	yelp = os.popen("yelp file://"+os.path.dirname(__file__)+"/docs/gladex-manual.xml")
	os.unlink(yelp)
#

def on_glade_file_chooser_selection_changed(widget, data, wtree):
	gladefile = widget.get_filename()
	if gladefile is not None and gladefile.find(".glade") < 0:
		show_message("This is not a glade file")
		widget.set_filename("")
	
	if global_data["home"] == global_data["tree"].get_widget("output_directory_chooser").get_filename() and gladefile!=None:
		f = gladefile[:gladefile.rfind("/")]
		global_data["tree"].get_widget("output_directory_chooser").set_filename(f)
		
	plug = get_selected_plugin()
	if plug == None: return None
	else:
		try:
			plug.on_glade_file_change(widget)
		except:
			return None
#

def on_output_directory_chooser_selection_changed(widget, data, wtree):
	plug = get_selected_plugin()
	if plug == None: return None
	else:
		try:
			plug.on_directory_change(widget)
		except:
			return None
	pass
